﻿/// <typ>SimpleXmlFile</typ>
/// <file>SimpleXmlFile.cs</file>
/// <create>14.09.2011 23:11:31</crate>
/// <author>Wolfgang Stelzhammer</author>
///
/// <summary>
///
/// </summary>
/// <copyright>Copyright © 2008-2011 Wolfgang Stelzhammer, All rights reserved!</copyright>
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using StorageHelper.StorageBase;
using StorageHelper.StorageLib;

namespace StorageHelper.StorageEngines
{
		public class SimpleXmlFile : StorageEngine
		{
				public override void Save(StorageHelperType sender, StorageHelperEventArgs e)
				{
						Init();
						if (e.CurrentObjectState.isNew)
								sender.Id = getNextAutoIncrement();

						writeToFile(getFileName(sender), e.CurrentObjectState.PropertiesContent.toXmlSerialize());

						e.CurrentObjectState.isNew = false;
				}

				public static int activeLockedThreads = 0;
				public static int notFinished = 0;

				private void writeToFile(string filename, string Content)
				{
						bool fileLocked = true;
						bool loked = false;
						notFinished++;

						do
						{
								try
								{
										File.WriteAllText(filename, Content);
										fileLocked = false;
										notFinished--;
								}
								catch (IOException)
								{
										if (!loked)
												activeLockedThreads++;
										loked = true;
										fileLocked = true;
										//Console.WriteLine(filename + " => locked!");
										//if (notFinished > 10)
										//  Thread.Sleep(20);
										//else if (notFinished < 20)
										//  Thread.Sleep(30);
										//else
										//  Thread.Sleep(40);

										Thread.Sleep(notFinished * 16);

										//else if (activeLockedThreads < 15)
										//  Thread.Sleep(1000);
								}
						} while (fileLocked);
						if (loked)
								activeLockedThreads--;
				}

				private string getFileName(StorageHelperType sender)
				{
						Init();

						string UniquID = sender.UniqID;

						return string.Format(fileFormat, dirStruct[DataDir], UniquID, fileExtension);
				}

				public override void Delete(StorageHelperType sender, StorageHelperEventArgs e)
				{
						Init();

						if (File.Exists(getFileName(sender)))
								File.Delete(getFileName(sender));

						sender.Clear();
				}

				private bool inited = false;

				private string dirName = string.Empty;

				private Dictionary<string, string> dirStruct = new Dictionary<string, string>();

				public SimpleXmlFile()
				{
				}

				private const string DataDir = "DATA";
				private const string AutoinrementDir = "AUTOINCREMENT";
				private const string IndexDir = "INDEX";
				private const string fileStdExtension = "data";
				private const string fileFormat = @"{0}{1}.{2}";

				private string fileExtension = fileStdExtension;

				public void Init()
				{
						if (!inited)
						{
								if (!TypeInfo.ConnectionString.EndsWith("\\"))
										TypeInfo.ConnectionString += "\\";

								if (!Directory.Exists(TypeInfo.ConnectionString))
										throw new ConnectionStringNotExistingException(_ConnectionString: TypeInfo.ConnectionString, _engine: this);
								dirName = TypeInfo.ConnectionString + TypeInfo.TypeCollectionName + "\\";

								dirStruct.Clear();
								dirStruct.Add(DataDir, dirName + DataDir + "\\");
								dirStruct.Add(AutoinrementDir, dirName + AutoinrementDir + "\\");
								dirStruct.Add(IndexDir, dirName + IndexDir + "\\");
								foreach (var dir in dirStruct)
								{
										if (!Directory.Exists(dir.Value))
												Directory.CreateDirectory(dir.Value);
								}

								if (CustonConfig != null && CustonConfig.ContainsKey("FileExtension"))

										fileExtension = CustonConfig["FileExtension"].ToString().Replace(".", "");

								inited = true;
						}
				}

				private const string AutoincrementFileName = "LastInsertedId.txt";

				private int getStdAutoI(string file)
				{
						File.WriteAllText(file, "1");
						return 1;
				}

				public override void GetAllContent(ConcurrentObservableDictionary<string, TypeContent> ContentCollection)
				{
						Init();

						string[] files = Directory.GetFiles(dirStruct[DataDir]);

						foreach (string f in files)
						{
								string renderedFilename = stripFileName(f);

								ContentCollection[renderedFilename] = TypeContent.fromXmlDeserialize(File.ReadAllText(f));
						}
				}

				private string stripFileName(string name)
				{
						FileInfo fi = new FileInfo(name);

						string ret = fi.Name;
						ret = ret.Replace(fi.Extension, "");

						return ret;
				}

				public override int getNextAutoIncrement()
				{
						Init();

						string file = dirStruct[AutoinrementDir] + AutoincrementFileName;

						if (File.Exists(file))
						{
								try
								{
										int lastID = Convert.ToInt32(File.ReadAllText(file));
										lastID++;
										File.WriteAllText(file, lastID.ToString());
										return lastID;
								}
								catch (FormatException)
								{
										return getStdAutoI(file);
								}
						}
						else
						{
								return getStdAutoI(file);
						}
				}
		}
}