﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using StorageHelper.StorageLib;

namespace StorageHelperTest.StorageLib
{
		/// <summary>
		///Dies ist eine Testklasse für "TypeContentTest" und soll
		///alle TypeContentTest Komponententests enthalten.
		///</summary>
		[TestClass()]
		public class TypeContentTest
		{
				private TestContext testContextInstance;

				/// <summary>
				///Ruft den Testkontext auf, der Informationen
				///über und Funktionalität für den aktuellen Testlauf bietet, oder legt diesen fest.
				///</summary>
				public TestContext TestContext
				{
						get
						{
								return testContextInstance;
						}
						set
						{
								testContextInstance = value;
						}
				}

				#region Zusätzliche Testattribute

				//
				//Sie können beim Verfassen Ihrer Tests die folgenden zusätzlichen Attribute verwenden:
				//
				//Mit ClassInitialize führen Sie Code aus, bevor Sie den ersten Test in der Klasse ausführen.
				//[ClassInitialize()]
				//public static void MyClassInitialize(TestContext testContext)
				//{
				//}
				//
				//Mit ClassCleanup führen Sie Code aus, nachdem alle Tests in einer Klasse ausgeführt wurden.
				//[ClassCleanup()]
				//public static void MyClassCleanup()
				//{
				//}
				//
				//Mit TestInitialize können Sie vor jedem einzelnen Test Code ausführen.
				//[TestInitialize()]
				//public void MyTestInitialize()
				//{
				//}
				//
				//Mit TestCleanup können Sie nach jedem einzelnen Test Code ausführen.
				//[TestCleanup()]
				//public void MyTestCleanup()
				//{
				//}
				//

				#endregion Zusätzliche Testattribute

				#region InitData

				private static TypeContent getSampleType()
				{
						TypeContent cont = new TypeContent();

						return cont;
				}

				private static void getSammpleData(TypeContent data)
				{
						data.Add("Id", "1");
						data.Add("Name", "Test Name");
						data.Add("Sonst", "laskdjflaskdjfasklö asldkfj");
						data.Add("werfwoiehfwkj", "sdflökasdflkas");
				}

				public static string sampleXmlData = "<?xml version=\"1.0\" encoding=\"utf-16\"?>" + @"
<TypeContent>
  <f>
    <key>
      <string>Id</string>
    </key>
    <value>
      <string>1</string>
    </value>
  </f>
  <f>
    <key>
      <string>Name</string>
    </key>
    <value>
      <string>Test Name</string>
    </value>
  </f>
  <f>
    <key>
      <string>Sonst</string>
    </key>
    <value>
      <string>laskdjflaskdjfasklö asldkfj</string>
    </value>
  </f>
  <f>
    <key>
      <string>werfwoiehfwkj</string>
    </key>
    <value>
      <string>sdflökasdflkas</string>
    </value>
  </f>
</TypeContent>";

				#endregion InitData

				/// <summary>
				///Ein Test für "TypeContent-Konstruktor"
				///</summary>
				[TestMethod()]
				public void TypeContentConstructorTest()
				{
						TypeContent target = getSampleType();
						Assert.IsNotNull(target);
						Assert.AreEqual(target.Count, 0);
						getSammpleData(target);
						Assert.AreEqual(target.Count, 4);
				}

				/// <summary>
				///Ein Test für "fromXmlDeserialize"
				///</summary>
				[TestMethod()]
				public void fromXmlDeserializeTest()
				{
						string XmlString = sampleXmlData;
						TypeContent expected = getSampleType();
						getSammpleData(expected);

						TypeContent actual;
						actual = TypeContent.fromXmlDeserialize(XmlString);

						Assert.AreEqual(actual, expected, "Fehlerhafte deserialisierung");
				}

				/// <summary>
				///Ein Test für "fromXmlDeserializeStringEmty"
				///</summary>
				[TestMethod()]
				public void fromXmlDeserializeStringEmtyTest()
				{
						string XmlString = string.Empty;
						TypeContent expected = getSampleType();
						getSammpleData(expected);

						TypeContent actual;

						try
						{
								actual = TypeContent.fromXmlDeserialize(XmlString);
						}
						catch (DeserializeExeception de)
						{
								Assert.AreEqual(de.Failure, DeserializeExeception.TypeContentFailures.XmlStringEmty, "Keine String.Emty Fehlernachricht");
						}
						catch (Exception)
						{
								Assert.Fail("Unbehandelte Exception");
						}
				}

				/// <summary>
				///Ein Test für "fromXmlDeserializeNull"
				///</summary>
				[TestMethod()]
				public void fromXmlDeserializeNullTest()
				{
						string XmlString = null;
						TypeContent expected = getSampleType();
						getSammpleData(expected);

						TypeContent actual;

						try
						{
								actual = TypeContent.fromXmlDeserialize(XmlString);
						}
						catch (DeserializeExeception de)
						{
								Assert.AreEqual(de.Failure, DeserializeExeception.TypeContentFailures.XmlStringEmty, "Keine String.Emty Fehlernachricht");
						}
						catch (Exception)
						{
								Assert.Fail("Unbehandelte Exception");
						}
				}

				/// <summary>
				///Ein Test für "toXmlSerialize"
				///</summary>
				[TestMethod()]
				public void toXmlSerializeTest()
				{
						TypeContent target = getSampleType();
						getSammpleData(target);

						string expected = sampleXmlData;
						string actual;
						actual = target.toXmlSerialize();
						Assert.AreEqual(expected, actual);
				}

				/// <summary>
				///Ein Test für "Equals"
				///</summary>
				[TestMethod()]
				public void EqualsTest()
				{
						TypeContent t1 = getSampleType();
						getSammpleData(t1);

						TypeContent t2 = getSampleType();
						getSammpleData(t2);

						Assert.AreEqual(t1, t2, "Equals arbeitet nicht ordnungsgemäß");
				}

				/// <summary>
				///Ein Test für "NotEquals"
				///</summary>
				[TestMethod()]
				public void NotEqualsTest()
				{
						TypeContent t1 = getSampleType();
						getSammpleData(t1);

						TypeContent t2 = getSampleType();
						getSammpleData(t2);
						t2.Add("dfasdf", "sadfasdf");

						Assert.AreNotEqual(t1, t2, "Equals arbeitet nicht ordnungsgemäß");
				}

				/// <summary>
				///Ein Test für "GetHashCode"
				///</summary>
				[TestMethod()]
				public void GetHashCodeTest()
				{
						TypeContent target = getSampleType();
						getSammpleData(target);
						int expected = 1;
						int actual;
						actual = target.GetHashCode();

						Assert.AreEqual(expected, actual, "GetHashCode() gibt nicht die erwartete Id zurück.");

						target["Id"] = string.Empty;
						actual = target.GetHashCode();
						Assert.AreNotEqual(expected, actual, "Fehler bei String.Emty-Fallback");

						target["Id"] = null;
						actual = target.GetHashCode();
						Assert.AreNotEqual(expected, actual, "Fehler bei Null-Fallback");

						target.Remove("Id");
						actual = target.GetHashCode();
						Assert.AreNotEqual(expected, actual, "Fehler bei nicht vorhanden-Fallback");
				}
		}
}