﻿/// <typ>TypeConverter</typ>
/// <file>TypeConverter.cs</file>
/// <create>22.09.2011 01:06:11</crate>
/// <author>Wolfgang Stelzhammer</author>
///
/// <summary>
///
/// </summary>
/// <copyright>Copyright © 2008-2011 Wolfgang Stelzhammer, All rights reserved!</copyright>
using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace StorageHelper.StorageLib
{
		/// <summary>
		/// Hier werden nach möglichkeit alle (ausgenommen in From und To angeführt) von Lightswitch BasisTypen convertiert
		/// </summary>
		public static partial class TypeConverter
		{
				public const int StdConverterCount = 12;

				public delegate object ToDelegate(string FromString);
				public delegate string FromDelegate(object FromObject);

				private static List<string> _StdConverters = new List<string>();

				public static List<string> StdConverters
				{
						get
						{
								Contract.Ensures(_StdConverters.Count == StdConverterCount);
								if (_StdConverters.Count != StdConverterCount)
								{
										InitStdConverters();
										Contract.Assert(_StdConverters.Count == StdConverterCount);
								}
								else
										Contract.Assert(_StdConverters.Count == StdConverterCount);

								return _StdConverters;
						}
				}

				private static bool StdConvertersInited = false;

				private static void UpdateStdConverters()
				{
						Contract.Ensures(StdConverters.Count == StdConverterCount);
						if (!StdConvertersInited)
						{
								StdConvertersInited = true;
								_StdConverters.Clear();
								foreach (KeyValuePair<string, FromDelegate> item in FromDelegater)
								{
										_StdConverters.Add(item.Key);
								}

								Contract.Assert(StdConverters.Count == StdConverterCount);
						}
						else
								Contract.Assert(StdConverters.Count == StdConverterCount);
				}

				private static void InitStdConverters()
				{
						Contract.Ensures(StdConverters.Count == StdConverterCount);
						InInit = true;
						addConverter("Int32", To.Int32, From.Int32);
						addConverter("String", To.String, From.String);
						addConverter("Int16", To.Int16, From.Int16);
						addConverter("int", To.Int, From.Int);
						addConverter("Boolean", To.Boolean, From.Boolean);
						addConverter("DateTime", To.DateTime, From.DateTime);
						addConverter("Decimal", To.Decimal, From.Decimal);
						addConverter("Double", To.Double, From.Double);
						addConverter("Guid", To.Guid, From.Guid);
						addConverter("SByte", To.SByte, From.SByte);
						addConverter("Single", To.Single, From.Single);
						addConverter("TimeSpan", To.TimeSpan, From.TimeSpan);
						UpdateStdConverters();
						InInit = false;
				}

				private static Dictionary<string, ToDelegate> _ToDelegater = new Dictionary<string, ToDelegate>();

				public static Dictionary<string, ToDelegate> ToDelegater
				{
						get
						{
								Contract.Ensures(_ToDelegater.Count >= StdConverterCount);
								//if (_ToDelegater == null || _ToDelegater.Count < StdConverterCount)
								if (_ToDelegater.Count < StdConverterCount)
								{
										InitStdConverters();
										Contract.Assume(_ToDelegater.Count >= StdConverterCount);
								}
								else
										Contract.Assert(_ToDelegater.Count >= StdConverterCount);
								return _ToDelegater;
						}
				}

				private static Dictionary<string, FromDelegate> _FromDelegater = new Dictionary<string, FromDelegate>();

				public static Dictionary<string, FromDelegate> FromDelegater
				{
						get
						{
								Contract.Ensures(_FromDelegater.Count >= StdConverterCount);
								//if (_FromDelegater == null || _FromDelegater.Count < StdConverterCount)
								if (_FromDelegater.Count < StdConverterCount)
								{
										InitStdConverters();
										Contract.Assume(_FromDelegater.Count >= StdConverterCount);
								}
								else
										Contract.Assert(_FromDelegater.Count >= StdConverterCount);
								return _FromDelegater;
						}
				}

				public static void addConverter(string _TypeName, ToDelegate _customToDelegate, FromDelegate _customFromDelegate)
				{
						Contract.Requires(_TypeName != null);
						if (!StdConvertersInited && !InInit)
								InitStdConverters();
						if (_ToDelegater.ContainsKey(_TypeName) || _FromDelegater.ContainsKey(_TypeName))
								throw new TypeConverterException(TypeConverterException.ExceptionTypes.ContainsType, _TypeName);
						else
						{
								_ToDelegater.Add(_TypeName, _customToDelegate);
								_FromDelegater.Add(_TypeName, _customFromDelegate);
						}
				}

				public static void removeConverter(string _TypeName)
				{
						Contract.Requires(_TypeName != null);
						if (!StdConvertersInited && !InInit)
								InitStdConverters();
						if (!_ToDelegater.ContainsKey(_TypeName) && !_FromDelegater.ContainsKey(_TypeName))
								throw new TypeConverterException(TypeConverterException.ExceptionTypes.notContainsType, _TypeName);
						else if (_StdConverters.Contains(_TypeName))
								throw new TypeConverterException(TypeConverterException.ExceptionTypes.isStdType, _TypeName);
						else
						{
								_ToDelegater.Remove(_TypeName);
								_FromDelegater.Remove(_TypeName);
						}
				}

				private static bool InInit = false;
		}
}